/*
 * BeOpen - a compressed file opener for BeOS
 *
 * Written by Paul Clip, paul@clip.org
 *
 * Drag any of the following type of files onto the BeOpen icon
 * and they will be un-whatever'ed into a /tmp directory and
 * the Browser will bring up the directory in a window
 *
 * Supported file extensions are:
 * 
 * .gz		.Z		.tar.Z		.tar.gz
 * .tar		.shar	.shar.z		.shar.GZ
 * .uu		.UU		.sit		.zip
 * .cpt		.bin
 *
 * Note: I unfortunately don't have access to a BeBox anymore
 * 		 so this is no longer supported software. Have fun with it.
 */

#include "BeOpen.h"


void show_alert(char *);
long run_command(char *, char *, char *, char *);
void create_directory(char *, record_ref *);


void main() {

	BeOpen *app = new BeOpen();
	app->Run();
	delete app;
	
}



BeOpen:: BeOpen() : BApplication('PLC0') {

	BWindow *fooWin = new BWindow(BRect(0,0,1,1), "w", B_BORDERED_WINDOW, 0);
	BView *fooView = new BView(BRect(0,0,1,1), "v", B_FOLLOW_NONE, 0);
	fooWin->AddChild(fooView);

	BPopUpMenu *menu = new BPopUpMenu("Main", FALSE, FALSE);
	BMenuItem *item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');
	menu->AddItem(item);
	SetMainMenu(menu);

}


void BeOpen:: ArgvReceived(int argc, char **argv) {
	
	record_ref ref;
	app_info info;
	
	if (be_app->GetAppInfo(&info) != B_NO_ERROR) {
		puts("Error: Couldn't get Application Info");
		return;
	}
	while (argc-- > 1) {
		printf ("Handling: %s\n", argv[argc]);
		if (get_ref_for_path(argv[argc], &ref) != B_NO_ERROR) {
			printf ("Error: unable to open %s\n", argv[argc]);
			continue;
		}
		if (!does_ref_conform(ref, "File")) {
			printf ("Error: %s is not a file\n", argv[argc]);
			continue;
		}
		HandleFile(&ref);
	}
	
}


void BeOpen:: RefsReceived(BMessage *message) {

	ulong type;
	long count;
	
	message->GetInfo("refs", &type, &count);
	while (count--) {
		record_ref *entry = new record_ref;
		*entry = message->FindRef("refs", count);
		if (!does_ref_conform(*entry, "File")) {
			show_alert("Not a file!");
			continue;
		}
		HandleFile(entry);
	}

}


// When ReadyToRun is called we should already have processed
// all the files we were given, so we just die. Sad, really.
void BeOpen:: ReadyToRun() {

	PostMessage(B_QUIT_REQUESTED, NULL);

}


void BeOpen:: MessageReceived(BMessage *message) {

	switch (message->what) {
		default:
			BApplication::MessageReceived(message);
			break;
	}

}

#define MAX_FILENAME_LENGTH		1000

extern char ** environ;

void BeOpen:: HandleFile(record_ref *ref) {

	BFile file, app;
	BDirectory parent, utils;
	app_info info;
	record_ref dirref;
	char filename[MAX_FILENAME_LENGTH],
		 scriptname[MAX_FILENAME_LENGTH],
		 scriptdir[MAX_FILENAME_LENGTH],
		 dirname[MAX_FILENAME_LENGTH];
	long res;

	file.SetRef(*ref);
	file.GetPath(filename, MAX_FILENAME_LENGTH);

	if (be_app->GetAppInfo(&info) != B_NO_ERROR) {
		show_alert("Couldn't find App Info. Dying.");
		return;
	}
	
	app.SetRef(info.ref);
	if (app.GetParent(&parent) != B_NO_ERROR) {
		show_alert("Couldn't find parent directory. Dying.");
		return;
	}
	
	if (parent.GetDirectory("BeOpenUtils", &utils) != B_NO_ERROR) {
		show_alert("Couldn't find BeOpenUtils directory. Dying.");
		return;
	}
	
	utils.GetPath(scriptdir, MAX_FILENAME_LENGTH);
	
	if (utils.GetFile("BeOpenScript", &app) != B_NO_ERROR) {
		show_alert("Couldn't find BeOpenScript. Dying.");
		return;
	}
	
	app.GetPath(scriptname, MAX_FILENAME_LENGTH);
	
	create_directory(dirname, &dirref);
	
	if (run_command(scriptname, filename, dirname, scriptdir) != 0) {
//		char str[1000], name[B_FILE_NAME_LENGTH+1];
//		file.GetName(name);
//		sprintf(str, "Error processing %s!", name);
//		show_alert("Error processing file");
//		show_alert(str);
//		return;
	} 
	
	BMessage *tobrowser = new BMessage();
	BMessenger *ponyexpress = new BMessenger('SHRK', -1);
	tobrowser->what = 'Bopn';
	tobrowser->AddRef("refs", dirref);
	ponyexpress->SendMessage(tobrowser);
	
}


long run_command(char *com, char *fn, char *dir, char *scriptd) {

	BFile exec_file;
	record_ref exec_ref;
	char **argv;
	long argc;
	thread_id exec_thread;
	long return_value;

	get_ref_for_path(com, &exec_ref);
	exec_file.SetRef(exec_ref);

	argc = 5;
	argv = (char **) malloc(sizeof(char) * (argc + 1));

	argv[0] = strdup("/boot/bin/sh");
	argv[1] = strdup(com);
	argv[2] = strdup(fn);
	argv[3] = strdup(dir);
	argv[4] = strdup(scriptd);
	argv[5] = NULL;
		
	exec_thread = load_executable(&exec_file, argc, argv, environ);
	
	if (wait_for_thread(exec_thread, &return_value) != B_NO_ERROR) {
		return_value = -1;
	}
	return return_value;
	
}


void show_alert(char *str) {

	BAlert *info= new BAlert("", str,
				"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT);
	info->Go();

}

void create_directory(char *dirname, record_ref *ref) {

	record_ref tmpref;
	BDirectory bdir;
	BDirectory newdir;
	int i = 0;
	
	get_ref_for_path("/boot/tmp", &tmpref);
	bdir.SetRef(tmpref);
	
	sprintf(dirname, "BeOpen_%d", i++);
	while (bdir.Create(dirname, &newdir) != B_NO_ERROR) {
		sprintf(dirname, "BeOpen_%d", i++);
	}
	
	*ref = newdir.Record()->Ref();

}
